#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
#
# seb@sisecurite.fr
#

import os
import sys
import signal
import subprocess
from optparse import OptionParser

# nt hash max plaintext length
NT_LEN = 32

# Must be synchronized with charset.h
CHARSETS = {'UPPER_ALPHA': 1,
            'LOWER_ALPHA': 2,
            'NUMERIC': 4,
            'SYMBOL14': 8,
            'ACCENT4': 16,
            'MIXED_ALPHA': 3,
            'MIXED_ALPHANUM': 7}


class Instance(object):
    def __init__(self, options):
        self.options = options
        o = self.options

        self.pos = o.start
        if o.end == 0:
            o.end = NT_LEN

        cmd = [o.binary]

        if o.verbose:
            cmd.append('-v')

        mode = None
        if o.dummy:
            mode = '-d'
        assert mode is not None
        cmd.append(mode)

        ct = None
        if o.ciphertext:
            ct = '-c %s' % o.ciphertext
        if o.filename:
            ct = '-f %s' % o.filename
        assert ct is not None
        cmd.append(ct)

        cmd.append('-a %d' % o.charset)
        self.cmd = cmd

    def __iter__(self):
        return self

    def next(self):
        if self.pos < self.options.end:
            cmd = self.cmd[:]
            cmd.append('-s %d' % self.pos)
            cmd.append('-e %d' % self.pos)
            self.pos += 1
            return cmd
        raise StopIteration


class Options(object):
    def __init__(self, raw, binary):
        self.raw = raw
        if not binary.startswith(os.sep):
            self.binary = os.path.join(os.getcwd(), binary)
        else:
            self.binary = binary
        assert os.access(self.binary, os.X_OK)

    def _HandleCharset(self, l):
        code = 0
        if l:
            cs_list = l.split(',')
            for cs in cs_list:
                val = CHARSETS.get(cs)
            if val:
                code |= val
            else:
                print >> sys.stderr, "Unknown charset", cs
        else:
            code = CHARSETS['MIXED_ALPHANUM']

        self.charset = code

    def Refine(self):
        o = self.raw
        self.verbose = int(o.verbose)
        self.dummy = int(o.dummy)
        assert self.dummy
        self._HandleCharset(o.charset)
        self.filename = o.filename
        self.ciphertext = o.ciphertext
        self.start = o.start
        self.end = o.end


def ParseOpt():
    usage = "usage: %prog [options]"

    parser = OptionParser(usage=usage)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      dest="verbose", help="Verbose mode")

    parser.add_option("-d", "--dummy", action="store_true", default=False,
                      dest="dummy", help="Activate dummy mode")

    parser.add_option("-a", "--charset", metavar="charset[,charset]",
                      dest="charset", action="store", type="string",
                      help=("A comma-separated list of charsets, accepted "
                            "values are: UPPER_ALPHA, LOWER_ALPHA, "
                            "NUMERIC, SYMBOL14, ACCENT4, MIXED_ALPHA, "
                            "MIXED_ALPHANUM"))

    parser.add_option("-f", "--file", action="store", type="string",
                      dest="filename",
                      help=("List of ciphertexts (string hex hashes "
                            "separated with \\n) to crack)"))

    parser.add_option("-c", "--ciphertext", action="store", type="string",
                      dest="ciphertext",
                      help="Ciphertext (string hex hash) to crack")

    parser.add_option("-s", "--start", action="store", type="int",
                      dest="start",
                      help="Generate plaintexts from this plaintext length")

    parser.add_option("-e", "--end", action="store", type="int",
                      dest="end",
                      help=("Generate plaintexts to this plaintext "
                            "length (included)"))

    (options, args) = parser.parse_args()
    return options


def Main():
    raw_options = ParseOpt()
    options = Options(raw_options, 'nt_spulet_crack')
    options.Refine()

    childs = []
    for cmd in Instance(options):
        if options.verbose:
            print 'Running', ' '.join(cmd)
        childs.append(subprocess.Popen(cmd))

    try:
        while childs:
            child = childs[-1]
            child.wait()
            childs.pop()
    except KeyboardInterrupt:
        while childs:
            try:
                pid = childs.pop().pid
                os.kill(pid, signal.SIGKILL)
            except OSError, err:
                #print >> sys.stderr, err
                pass


if __name__ == "__main__":
    Main()
